package writer2latex.xhtml;

import java.util.LinkedList;



import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import writer2latex.office.XMLString;
import writer2latex.util.Misc;


//LinkedList<String>   stringList = new LinkedList<String>();
public class GreenstoneTags {

	private static final String NONE = "none";
	private static final String SECTIONS = "sections";
	private static final String DIV = "div";
	
	
	private static LinkedList<Integer> headerStack = new LinkedList<Integer>();
	private static boolean pageOpened = false; 
	//headings none
	private static String headingTags = "headings";
	//sections div none
	private static String pageTags = "sections";

	protected static Node processHeading(Node currentNode, Node hnode, int pageNum) {
		
	
		if (headingTags.equals(NONE)){
			return hnode;
		}
		
		String sLevel = Misc.getAttribute(currentNode, XMLString.TEXT_OUTLINE_LEVEL);
		//If this heading contain outline-level
		if (sLevel != null && !sLevel.isEmpty()) {
			int nLevel = Integer.parseInt(sLevel);
			if (pageOpened) {
				hnode = closePage(hnode);
			}
			
			closeHeadingSections(hnode, nLevel);
			Node nextNode = currentNode.getNextSibling();
			//Useless to open section if we are at end of the document
			if (nextNode != null){
				String title = getTitle(currentNode); 
				//Open Heading section
				openSection(hnode, title);
				headerStack.offerFirst(Integer.parseInt(sLevel));
				//If next node is not a heading element with outline number and tags type is headings-pages 
				//Then open paged section
				if (!(
					nextNode.getNodeType() == Node.ELEMENT_NODE 
					&& nextNode.getNodeName().equals(XMLString.TEXT_H)
					&& Misc.getAttribute(nextNode, XMLString.TEXT_OUTLINE_LEVEL) != null
					&& !Misc.getAttribute(nextNode, XMLString.TEXT_OUTLINE_LEVEL).isEmpty()
					)) {
					hnode = openPage(hnode, pageNum);
				}
			}	
				
		} 
		return hnode;
	}
	protected static Node processPageBreak(Node currentNode, Node hnode, Integer pageNum){
		if (pageTags.equals(NONE)){
			return hnode;
		}
		
		if ( !( currentNode.getNodeType() == Node.ELEMENT_NODE 
				&& Misc.getAttribute(currentNode, XMLString.TEXT_OUTLINE_LEVEL) != null
				&& !Misc.getAttribute(currentNode, XMLString.TEXT_OUTLINE_LEVEL).isEmpty()
				)
				) {
			if (pageOpened) {
				hnode = closePage(hnode);
			}
			hnode = openPage(hnode, pageNum);
		} 
		return hnode;
	}
	//Method to open main document tag
	protected static Node StartDocument(Node hnode, String title, String heading, String pages, int pageNum){
		headingTags = heading;
		pageTags = pages;
		
		if (headingTags.equals(NONE) && pageTags.equals(NONE)){
			return hnode;
		}
		if(headingTags.equals(SECTIONS)){
			//Create global section
			openSection(hnode, title);
		}
		hnode = openPage(hnode, pageNum);
		
		return hnode;
	}
	//Method to close open tags at the end of the document
	protected static Node endDocument(Node hnode){
		if (headingTags.equals(NONE) && pageTags.equals(NONE)){
			return hnode;
		}
		if (pageOpened){
			hnode = closePage(hnode);
		}
		if (headingTags.equals(SECTIONS)){
			closeHeadingSections(hnode, 0);
			//Close global section
			closeSection(hnode);
		} 
		
		return hnode;
	}
	
	private static Node openPageDiv(Node hnode,int pageNum){
		Document doc = hnode.getOwnerDocument();
		Element openBlock = (Element) doc.createElement("div");
		openBlock.setAttribute("class", "pageNum");
		openBlock.setAttribute("page", Integer.toString(pageNum));
		// insert open section comment before header node
		hnode.appendChild((Node)openBlock);
		hnode = openBlock;
		return openBlock;
	}
	
	private static Node exitPageDiv(Node hnode){
		hnode = hnode.getParentNode();
		return hnode;
	}
	
	private static String getTitle(Node currentNode) {
		Node content = currentNode.cloneNode(true);
		NodeList contentNodes = content.getChildNodes();
		String title = null;

		int i = 0;
		while (i < contentNodes.getLength()) {
			Node child = contentNodes.item(i);
			if (child.getNodeType() == Node.ELEMENT_NODE){
				if (child.getNodeName().equals(XMLString.TEXT_TAB) ||
					child.getNodeName().equals(XMLString.TEXT_LINE_BREAK)	){
					Document doc = child.getOwnerDocument();
					Node testSpace = doc.createTextNode(" ");
					content.insertBefore(testSpace, child);
					content.removeChild(child);
				}
			}
				i++;
		} 
		
		title = content.getTextContent();
		
		return title;
	}

	
	private static void openPageSection(Node hnode, Integer pageNum){
		Document doc = hnode.getOwnerDocument();
		String commentText = "<Section>\n<Description>\n<Metadata name=\"Title\">" + pageNum
				+ "</Metadata>\n<Metadata name=\"Page\">" + pageNum + "</Metadata>\n</Description>";
		Node openSection = doc.createComment(commentText);
		// insert open section comment before header node
		hnode.appendChild(openSection);
	}
	private static void openSection(Node hnode, String title){
		Document doc = hnode.getOwnerDocument();
		String commentText = "<Section>\n<Description>\n<Metadata name=\"Title\">" + title
				+ "</Metadata>\n</Description>";
		Node openSection = doc.createComment(commentText);
		// insert open section comment before header node
		hnode.appendChild(openSection);
	}

	private static void closeSection(Node hnode){
		Document doc = hnode.getOwnerDocument();
		String commentText = "</Section>";
		Node closeSection = doc.createComment(commentText);
		//insert open section comment before header node
		hnode.appendChild(closeSection);
	}
	
	private static void closeHeadingSections(Node hnode, int nLevel){
		if (headerStack.isEmpty()) {
			return;
		}
		//Close all sections with level less than current
		while (nLevel <= headerStack.peek()) {
			closeSection(hnode);
			headerStack.poll();
			if (headerStack.isEmpty()) {
				break;
			}
		}
	}
	private static Node closePage(Node hnode){
		if (pageTags.equals(SECTIONS)){
			closeSection(hnode);
		}
		else if (pageTags.equals(DIV)){
			hnode = exitPageDiv(hnode);
		}
		pageOpened = false;
		return hnode;
	}
	private static Node openPage(Node hnode, Integer pageNum){
		if (pageTags.equals(SECTIONS)){
			openPageSection(hnode, pageNum);
		} 
		else if (pageTags.equals(DIV)){
			hnode = openPageDiv(hnode, pageNum);
		}
		pageOpened = true;
		return hnode;
	}
	
}
	